/**
 * js logic specific to opening and closing the panels from the site header elements
 */

import ee from "./event-emitter";
import EVENT_NAMES from "./event-names";
import { ariaToggle, toArray } from "./dom-methods";

const panelSelector = "#o-panel";
const panelCatSelector = ".o-panel-categories";
const panelSearchSelector = ".o-panel-search";
const panelBgHitSelector = ".o-panel-bg-area";
const closeControlSelector = "#panel-close-button";
const panelOpenedClass = "o-panel--opened";
const panelOpeningClass = "o-panel--opening";
const panelClosingClass = "o-panel--closing";
const panelCatsOpenClass = "o-panel-categories--opened";
const panelSearchOpenClass = "o-panel-search--opened";
const delayTime = 10;
const animationTime = delayTime + 250;

let panel = null;
let panelCategories = null;
let panelSearch = null;
let panelBgHit = null;
let closeControl = null;
let panelOpeningTimeout = null;
let panelOpenedTimeout = null;
let panelClosedTimeout = null;

/**
 * @method init
 * @description init method, adds event listener for scroll and resize
 * @return {undefined}
 */
export default function init() {
  panel = document.querySelector(panelSelector);
  panelCategories = document.querySelector(panelCatSelector);
  panelSearch = document.querySelector(panelSearchSelector);
  closeControl = document.querySelector(closeControlSelector);

  enableListeners(true);
}

/**
 * @method enableListeners
 * @description adds event listeners to panel buttons
 * @param {boolean} toggleOn - whether to turn click listeners on or off
 * @return {undefined}
 */
export function enableListeners(toggleOn) {
  if (toggleOn) {
    ee.on(EVENT_NAMES.panel.openCategories, toggleCategoriesPanel);
    ee.on(EVENT_NAMES.panel.openSearch, toggleSearchPanel);
    closeControl && closeControl.addEventListener("click", panelClose);
  } else {
    ee.off(EVENT_NAMES.panel.openCategories, toggleCategoriesPanel);
    ee.off(EVENT_NAMES.panel.openSearch, toggleSearchPanel);
    closeControl && closeControl.removeEventListener("click", panelClose);
  }
}

/**
 * @method toggleCategoriesPanel
 * @description on menu clicked
 * @return {undefined}
 */
export function toggleCategoriesPanel() {
  const isMenuOpen = panel.classList.contains(panelOpenedClass) || panel.classList.contains(panelOpeningClass);
  const isCatOpen = panelCategories.classList.contains(panelCatsOpenClass);

  isMenuOpen && isCatOpen ? panelClose() : categoriesOpen();
}

/**
 * @method toggleSearchPanel
 * @description adds open class
 * @return {undefined}
 */
export function toggleSearchPanel() {
  const isMenuOpen = panel.classList.contains(panelOpenedClass) || panel.classList.contains(panelOpeningClass);
  const isSearchOpen = panelSearch.classList.contains(panelSearchOpenClass);

  isMenuOpen && isSearchOpen ? panelClose() : searchOpen();
}

/**
 * @method categoriesOpen
 * @description adds open class specific to categories
 * @return {undefined}
 */
export function categoriesOpen() {
  panelOpen("categories");
}

/**
 * @method searchOpen
 * @description adds open class specific to search
 * @return {undefined}
 */
export function searchOpen() {
  panelOpen("search");
}

/**
 * @method panelOpen
 * @description opens the panel and dynamically emits the component name for analytics tracking purposes
 * @param {string} [componentName = categories] one of two string values for opening the panel: "categories" or "search"
 * @return {undefined}
 */
export function panelOpen(componentName = "categories") {
  // remove hidden should ALWAYS be called first without a timeout
  // if you don't separate this with a timeout, you won't see any animations
  panel.removeAttribute("hidden");

  clearTimeout(panelOpeningTimeout);
  panelOpeningTimeout = setTimeout(panelOpening.bind(this, componentName), delayTime);

  clearTimeout(panelOpenedTimeout);
  panelOpenedTimeout = setTimeout(panelOpened.bind(this, componentName), animationTime);
}

/**
 * @method panelOpening
 * @description starts the animation for fading in panel
 * @param {string} componentName, one of two string values for opening the panel: "categories" or "search"
 * @return {undefined}
 */
function panelOpening(componentName) {
  // the `emit` method is causing a considerable delay, so this should be called at the top of the method
  ee.emit(`panel:${componentName}-opening`);
  ee.emit(EVENT_NAMES.panel.opening);

  setPanelTransitionClass(panelOpeningClass);
  if (componentName === "categories") {
    panelCategories.classList.add(panelCatsOpenClass);
    panelSearch.classList.remove(panelSearchOpenClass);
  } else {
    panelCategories.classList.remove(panelCatsOpenClass);
    panelSearch.classList.add(panelSearchOpenClass);
  }

  ariaToggle(".c-control--menu-button .c-control__button", `close ${componentName}`, true);
  navigationFade(true);
}

/**
 * @method panelOpened
 * @description after the animation is finished the background becomes clickable and the scroll is locked through the scrollmanager
 * @param {string} componentName, one of two string values for opening the panel: "categories" or "search"
 * @return {undefined}
 */
function panelOpened(componentName) {
  ee.emit(`panel:${componentName}-opened`);
  ee.emit(EVENT_NAMES.panel.opened);

  setPanelTransitionClass(panelOpenedClass);
  createTransparentClickableBG();
}

/**
 * @method panelClose
 * @description removes open classes
 * @return {undefined}
 */
export function panelClose() {
  ee.emit(EVENT_NAMES.panel.closing);
  ee.emit(EVENT_NAMES.panel.closingCategories);
  ee.emit(EVENT_NAMES.panel.closingSearch);

  setPanelTransitionClass(panelClosingClass);
  ariaToggle(".c-control--menu-button .c-control__button", "open navigation", false);
  ariaToggle(".c-control--search-button .c-control__button", "open search", false);
  navigationFade(false);
  removeBgPanel();

  clearTimeout(panelClosedTimeout);
  panelClosedTimeout = setTimeout(() => {
    setPanelTransitionClass(null);
    panelCategories.classList.remove(panelCatsOpenClass);
    panelSearch.classList.remove(panelSearchOpenClass);

    panel.setAttribute("hidden", "hidden");

    ee.emit(EVENT_NAMES.panel.closed);
    ee.emit(EVENT_NAMES.panel.closedSearch);
    ee.emit(EVENT_NAMES.panel.closedCategories);
  }, animationTime);
}

/**
 * @method setPanelTransitionClass
 * @description swaps out panel transition/state class and adds a new class
 * @param {string|null} [className = null] class name to swap in
 * @return {undefined}
 */
export function setPanelTransitionClass(className = null) {
  panel.classList.remove(panelOpeningClass, panelOpenedClass, panelClosingClass);
  className && panel.classList.add(className);
}

/**
 * @method createTransparentClickableBG
 * @description adds the hidden background element
 * @return {undefined}
 */
export function createTransparentClickableBG() {
  if (!document.querySelector(".o-panel-bg-area")) {
    const divPanelBG = document.createElement("div");
    divPanelBG.classList.add("o-panel-bg-area");
    document.body.appendChild(divPanelBG);
    panelBgHit = document.querySelector(panelBgHitSelector);
    panelBgHit.addEventListener("click", panelClose);
  } else {
    document.querySelector(panelBgHitSelector) && document.querySelector(panelBgHitSelector).classList.remove("o-panel-bg-area--closed");
  }
}

/**
 * @method removeBgPanel
 * @description removes the hidden background element
 * @return {undefined}
 */
export function removeBgPanel() {
  document.querySelector(panelBgHitSelector) && document.querySelector(panelBgHitSelector).classList.add("o-panel-bg-area--closed");
}

/**
 * @method getHeaderElement
 * @description gets the current header element
 * @return {HTMLElement}
 */
export function getHeaderElement() {
  return document.querySelector(".c-header-nav") || document.querySelector(".c-header-title--article") || document.querySelector(".c-header-title--section");
}

/**
 * @method navigationFade
 * @description adds fade class to specific elements
 * @param {boolean} toggleOn - whether to turn fade in or fade out
 * @return {undefined}
 */
export function navigationFade(toggleOn) {
  const fadeClass = "c-control--disabled";
  const h2 = document.querySelector("h2");
  const scrollNav = document.querySelector(".c-scroll-menu__nav");
  const header = getHeaderElement();
  const allControlButtons = toArray(header.querySelectorAll(".c-control"));
  const ignoreClasses = ["c-control--menu-button", "c-control--search-button"];

  if (toggleOn) {
    allControlButtons.forEach((c) => {
      let notAnIgnoredClass = ignoreClasses.every((ignoreString) => {
        return !c.classList.contains(ignoreString);
      });

      notAnIgnoredClass && c.classList.add(fadeClass);
    });

    scrollNav && scrollNav.classList.add(fadeClass);
    h2 && h2.classList.add(fadeClass);
  } else {
    allControlButtons.forEach((c) => {
      c.classList.remove(fadeClass);
    });

    scrollNav && scrollNav.classList.remove(fadeClass);
    h2 && h2.classList.remove(fadeClass);
  }

}
