import ee from "./event-emitter";

const eventNamespace = "tgam-scroll-manager";

let layers = [];
let scrollPosition = null;
let scrollPositionSaved = false;

/**
 * @method registerLayer
 * @description registers a layer to keep track of how many elements are overlaid
 * @param {object} object - an object containing an "isOpen" method and a "lockEvent" and "unlockEvent" property
 * @property {function} object.isOpen - a method which determines whether the layer is enabled/visible on the page
 * @property {string} object.key - a unique id to register and unregister layers
 * @property {string} object.saveScrollEvent - event that will save the scroll position (e.g. before the position changes)
 * @property {string} object.lockEvent - name of event that will initiate scroll lock (e.g. when layer has finished opening)
 * @property {string} object.unlockEvent - name of event that will release scroll lock
 * @property {string} object.resetScrollEvent - name of event that will reset scroll (e.g. when layer has finished closing)
 * @return {undefined}
 */
export function registerLayer(object) {
  if (typeof object.key === "undefined") {
    console.error("scroll-manager: registerLayer method object parameter must have the 'key' property defined");
  }

  let layerObject = {
    key: object.key,
    isOpen: object.isOpen
  };

  layers.push(layerObject);

  if (object.saveScrollEvent) {
    ee.on(object.saveScrollEvent, saveScrollPosition);
  }

  if (object.lockEvent) {
    ee.on(object.lockEvent, lockScrolling);
  }

  if (object.unlockEvent) {
    ee.on(object.unlockEvent, unlockScrolling);
  }

  if (object.resetScrollEvent) {
    ee.on(object.resetScrollEvent, resetScrollTop);
  }
}

/**
 * @method unregisterLayer
 * @description TODO
 * @param {object} obj - an object with an "isOpen" method
 * @return {undefined}
 */
export function unregisterLayer(obj) {
  layers = layers.filter(layer => {
    if (layer.key === obj.key) {
      if (layer.saveScrollEvent) {
        ee.off(layer.saveScrollEvent, saveScrollPosition);
      }
      if (layer.lockEvent) {
        ee.off(layer.lockEvent, lockScrolling);
      }
      if (layer.unlockEvent) {
        ee.off(layer.unlockEvent, unlockScrolling);
      }
      if (layer.resetScrollEvent) {
        ee.off(layer.resetScrollEvent, resetScrollTop);
      }
      return false;
    }
    return true;
  });
}

/**
 * @method saveScrollPosition
 * @description called by "saveScrollEvent" event, will save position once, and will not be reset until the layer is closed
 * @return {undefined}
 */
export function saveScrollPosition() {
  if (!scrollPositionSaved) {
    scrollPosition = window.pageYOffset || document.documentElement.scrollTop;
    scrollPositionSaved = true;
  }
}

/**
 * @method lockScrolling
 * @description makes the body unscrollable
 * @return {undefined}
 */
export function lockScrolling() {
  saveScrollPosition();
  document.body.classList.add("u-no-scroll");
  ee.emit(`${eventNamespace}:locked`, [true]);
}

/**
 * @method unlockScrolling
 * @description when a layer has finished closing, check whether it's safe to allow the body to scroll again (all layers must be closed)
 * @return {boolean}
 */
export function unlockScrolling() {
  resetScrollTop();
}

/**
 * @method resetScrollTop
 * @description removes no scroll class and reset scroll position
 * @return {void}
 */
export function resetScrollTop() {
  // every layer should have a method `isOpen` and the value for all of them must be `false`
  if (allLayersClosed()) {
    document.body.classList.remove("u-no-scroll");
    ee.emit(`${eventNamespace}:locked`, [false]);
    // reset to previous scroll top position
    window.scrollTo(0, scrollPosition);
    scrollPositionSaved = false;
  }
}

/**
 * @method containsClasses
 * @description loops through the array of class names and check to see if the element currently has that class,
 *  if any class exists on the element then method will return TRUE, if none of the classes are found on the element then method returns FALSE
 * @param {HTMLElement} el - a DOM element
 * @param {array} list - an array of class names
 * @return {boolean}
 */
export function containsClasses(el, list) {
  return list.some((className) => el.classList.contains(className));
}

/**
 * @method allLayersClosed
 * @description loops through all layers and returns true if `isOpen` methods on every layer returns false
 * @return {boolean}
 */
function allLayersClosed() {
  return layers.every((layer) => layer.isOpen && !layer.isOpen());
}
